externdef IID_NULL:GUID
;externdef IID_IUnknown:GUID

include Office.inc
include Wordc.inc
include Word.inc

Object WordHost,, Primer
 RedefineMethod    Init,         Pointer
 BoundMethod       DocClose
 RedefineMethod    Done
 BoundMethod       DocOpen,      BSTR
 BoundMethod       DocSaveAs,    BSTR
 BoundMethod       DocReplace,   BSTR, BSTR

 DefineVariable    pIWordApp,    Pointer,  NULL
 DefineVariable    pIWordDoc,    Pointer,  NULL
ObjectEnd

if IMPLEMENT

.const

IID_Application GUID {000020970h,00000h,00000h,{0C0h,000h,000h,000h,000h,000h,000h,046h}}

.code

; 
; Method:    WordHost.Done
; Purpose:   Finalizes the Host object.
; Arguments: None.
; Return:    Nothing.

Method WordHost.Done, uses esi
local vArg01:VARIANT, vArg02:VARIANT, vArg03:VARIANT

	MakeVariantBool vArg01,FALSE                 ;SaveChanges
 	TypeMissing vArg02,vArg03

 	SetObject esi
 	ICall [esi].pIWordApp::_Application.Quit, addr vArg01, addr vArg02, addr vArg03
    ICall [esi].pIWordApp::_Application.Release
    ACall esi.Done
MethodEnd NONE

; 
; Method:    WordHost.Close
; Purpose:   Closes a given document.
; Arguments: None.
; Return:    Nothing.

Method WordHost.DocClose
    local vArg01:VARIANT, vArg02:VARIANT, vArg03:VARIANT

    MakeVariantBool vArg01,FALSE                ;SaveChanges
    TypeMissing vArg02,vArg03

    SetObject ecx
    ICall  [ecx].pIWordDoc::_Document.Close, addr vArg01, addr vArg02, addr vArg03
MethodEnd NONE

; 
; Method:    WordHost.Init
; Purpose:   Initializes the Host object and creates an Word Application instance.
; Arguments: -> Owner.
; Return:    Nothing.

Method WordHost.Init, uses esi, pOwner:Pointer
    local ClsID:GUID

    SetObject esi
    ACall esi.Init, pOwner

    invoke CLSIDFromProgID, $OfsCUStr("Word.Application"), addr ClsID
    .if SUCCEEDED(eax)
      invoke CoCreateInstance, addr ClsID, 0, CLSCTX_SERVER, offset IID_Application, \
                               addr [esi].pIWordApp
    .endif
MethodEnd NONE

; 
; Method:    WordHost.DocReplace
; Purpose:   Replace string in current workbook
; Arguments: Arg1: -> Pattern to find.
;            Arg2: -> Replacement string
; Return:    Nothing.

Method WordHost.DocReplace,, pFrom:BSTR,pTo:BSTR
	local pContent:Pointer, pFind:Pointer
	local vArg01:VARIANT, vArg02:VARIANT, vArg03:VARIANT, vArg04:VARIANT
	local vArg05:VARIANT, vArg06:VARIANT, vArg07:VARIANT, vArg08:VARIANT
	local vArg09:VARIANT, vArg10:VARIANT, vArg11:VARIANT, vArg12:VARIANT
	local vArg13:VARIANT, vArg14:VARIANT, vArg15:VARIANT, retval:BOOL

 	mov pContent, NULL
 	mov pFind,    NULL

  	SetObject ecx
 	ICall [ecx].pIWordDoc::_Document.get_Content, ADDR pContent
 	.if SUCCEEDED(eax)
	  ICall pContent::Range.get_Find, ADDR pFind
	  .if SUCCEEDED(eax)
	   ICall pFind::Find.ClearFormatting

	   MakeVariantBstr vArg01,pFrom
	   MakeVariantBstr vArg10,pTo
	   MakeVariantWord vArg11,wdReplaceAll

	   TypeMissing vArg02, vArg03, vArg04, vArg05, vArg06, vArg07, vArg08, vArg09
	   TypeMissing vArg12, vArg13, vArg14, vArg15

	   ICall pFind::Find.Execute,addr vArg01, addr vArg02, addr vArg03, addr vArg04,\
    	                         addr vArg05, addr vArg06, addr vArg07, addr vArg08,\
        	                     addr vArg09, addr vArg10, addr vArg11, addr vArg12,\
            	                 addr vArg13, addr vArg14, addr vArg15, addr retval
	   ICall pFind::Find.Release
	  .endif
	 ICall pContent::Range.Release
	.endif
MethodEnd NONE

; 
; Method:    WordHost.Open
; Purpose:   Opens a Workbook from a file.
; Arguments: Arg1: -> Filename.
; Return:    Nothing.

Method WordHost.DocOpen, uses esi, pFileName:BSTR
    local pDocuments:Pointer
    local vArg01:VARIANT, vArg02:VARIANT, vArg03:VARIANT, vArg04:VARIANT
    local vArg05:VARIANT, vArg06:VARIANT, vArg07:VARIANT, vArg08:VARIANT
    local vArg09:VARIANT, vArg10:VARIANT, vArg11:VARIANT, vArg12:VARIANT

    SetObject esi
    ICall [esi].pIWordApp::_Application.get_Documents, addr pDocuments
    .if SUCCEEDED(eax)
     MakeVariantBstr vArg01,pFileName        ;File name
     MakeVariantBool vArg03,FALSE            ;Read only
     MakeVariantBool vArg12,FALSE            ;Is Visible

     TypeMissing vArg02, vArg04, vArg05, vArg06, vArg07, vArg08, vArg09, vArg10, vArg11

     ICall pDocuments::Documents.Open, addr vArg01, addr vArg02, addr vArg03, addr vArg04, \
                                       addr vArg05, addr vArg06, addr vArg07, addr vArg08, \
                                       addr vArg09, addr vArg10, addr vArg11, addr vArg12, \
                                       addr [esi].pIWordDoc

      .if SUCCEEDED(eax)
        ICall [esi].pIWordDoc::_Document.Activate
      .endif
      ICall pDocuments::Documents.Release
    .endif
MethodEnd NONE

; 
; Method:    WordHost.SaveAs
; Purpose:   Saves a given Workbook.
; Arguments: Arg1 -> NewFileName
; Return:    Nothing.

Method WordHost.DocSaveAs,, pNewFileName:BSTR
    local vArg01:VARIANT, vArg02:VARIANT, vArg03:VARIANT, vArg04:VARIANT
    local vArg05:VARIANT, vArg06:VARIANT, vArg07:VARIANT, vArg08:VARIANT
    local vArg09:VARIANT, vArg10:VARIANT, vArg11:VARIANT

    MakeVariantBstr vArg01,pNewFileName                ;FileName
    MakeVariantDword vArg02,wdFormatDocument           ;FileFormat

    TypeMissing vArg03, vArg05, vArg07, vArg08, vArg09, vArg10, vArg11
    TypeMissing vArg04, vArg06                    ;Passwords read/write

    SetObject ecx
    ICall [ecx].pIWordDoc::_Document.SaveAs, addr vArg01, addr vArg02, addr vArg03, addr vArg04, \
                                      		 addr vArg05, addr vArg06, addr vArg07, addr vArg08, \
                                     		 addr vArg09, addr vArg10, addr vArg11
MethodEnd NONE

endif
